package com.onsiteservice.miniapp.service.address;

import com.onsiteservice.constant.enums.BizCodeEnum;
import com.onsiteservice.core.exception.ServiceException;
import com.onsiteservice.core.result.Result;
import com.onsiteservice.core.result.ResultGenerator;
import com.onsiteservice.dao.mapper.service.ServiceAreaMapper;
import com.onsiteservice.entity.address.ServiceAddress;
import com.onsiteservice.entity.area.ServiceArea;
import com.onsiteservice.miniapp.controller.address.dto.PageServiceAddressDTO;
import com.onsiteservice.miniapp.controller.address.dto.SaveServiceAddressDTO;
import com.onsiteservice.miniapp.controller.address.dto.UpdateServiceAddressDTO;
import com.onsiteservice.miniapp.controller.address.vo.ServiceAddressVO;
import com.onsiteservice.miniapp.mapper.address.ServiceAddressBizMapper;

import com.onsiteservice.dao.common.AbstractMapper;
import com.onsiteservice.dao.common.page.PageInfoVO;
import com.github.pagehelper.PageHelper;
import com.onsiteservice.util.AttrCopyUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Condition;

import javax.annotation.Resource;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static com.onsiteservice.core.result.ResultGenerator.success;

@Service
@Transactional(rollbackFor = Exception.class)
public class ServiceAddressService extends AbstractMapper<ServiceAddress> {


    @Resource
    private ServiceAddressBizMapper serviceAddressBizMapper;

    @Resource
    private ServiceAreaMapper serviceAreaMapper;


    /**
     * 分页查询列表
     */
    public Result<PageInfoVO<ServiceAddressVO>> getPage(PageServiceAddressDTO dto, Long uid) {
        PageHelper.startPage(dto.getPage(), dto.getSize());
        List<ServiceAddress> serviceAddressList = serviceAddressBizMapper.selectServiceAddressPage(dto, uid);
        List<ServiceAddressVO> serviceAddressVOList = serviceAddressList.parallelStream().map(e -> AttrCopyUtils.copy(e, new ServiceAddressVO())).collect(Collectors.toList());
        return ResultGenerator.success(new PageInfoVO<>(serviceAddressVOList));
    }


    public ServiceAddressVO selectById(Long id) {
        ServiceAddress serviceAddress = this.selectByPrimaryKey(id);
        return AttrCopyUtils.copy(serviceAddress, new ServiceAddressVO());
    }


    public List<ServiceAddressVO> selectByUserId(Long userId) {
        Condition condition = new Condition(ServiceAddress.class);
        condition.createCriteria().andEqualTo("account_no", userId);
        List<ServiceAddress> serviceAddresses = this.selectByCondition(condition);

        return serviceAddresses.parallelStream()
                .map(e -> AttrCopyUtils.copy(e, new ServiceAddressVO()))
                .collect(Collectors.toList());
    }


    public int save(SaveServiceAddressDTO dto, Long userId) {
        checkServiceArea(dto.getAreaId());

        ServiceAddress serviceAddress = AttrCopyUtils.copy(dto, new ServiceAddress());
        serviceAddress.setAccountNo(userId);
        return this.insertSelective(serviceAddress);
    }


    public int update(UpdateServiceAddressDTO dto) {
        checkServiceArea(dto.getAreaId());

        ServiceAddress serviceAddress = AttrCopyUtils.copy(dto, new ServiceAddress());
        return this.updateByPrimaryKey(serviceAddress);
    }

    /**
     * 判断区域存在
     *
     * @param areaId 区域id
     */
    private void checkServiceArea(long areaId){
        ServiceArea serviceArea = serviceAreaMapper.selectByPrimaryKey(areaId);
        if (Objects.isNull(serviceArea)) {
            throw new ServiceException(BizCodeEnum.SERVICE_AREA_NOT_EXIST);
        }
    }


}
