package com.onsiteservice.miniapp.service.order;

import com.github.pagehelper.PageHelper;
import com.onsiteservice.constant.enums.BizCodeEnum;
import com.onsiteservice.core.exception.ServiceException;
import com.onsiteservice.core.result.Result;
import com.onsiteservice.core.result.ResultGenerator;
import com.onsiteservice.dao.common.AbstractMapper;
import com.onsiteservice.dao.common.page.PageInfoVO;
import com.onsiteservice.entity.order.ServiceOrder;
import com.onsiteservice.miniapp.controller.address.vo.ServiceAddressVO;
import com.onsiteservice.miniapp.controller.order.dto.PageServiceOrderDTO;
import com.onsiteservice.miniapp.controller.order.dto.SaveServiceOrderDTO;
import com.onsiteservice.miniapp.controller.order.dto.ValuationServiceOrderDTO;
import com.onsiteservice.miniapp.controller.order.vo.ServiceOrderVO;
import com.onsiteservice.miniapp.mapper.order.ServiceOrderBizMapper;
import com.onsiteservice.miniapp.service.address.ServiceAddressService;
import com.onsiteservice.util.AttrCopyUtils;
import com.onsiteservice.util.RandomUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
@Transactional(rollbackFor = Exception.class)
public class ServiceOrderService extends AbstractMapper<ServiceOrder> {

    @Resource
    private ServiceOrderBizMapper serviceOrderBizMapper;

    @Resource
    private ServiceAddressService serviceAddressService;


    public ServiceOrderVO selectById(Long id) {
        // 订单信息
        ServiceOrder serviceOrder = this.selectByPrimaryKey(id);
        ServiceOrderVO serviceOrderVO = AttrCopyUtils.copy(serviceOrder, new ServiceOrderVO());

        // 订单地址信息
        ServiceAddressVO serviceAddressVO = serviceAddressService.selectById(serviceOrder.getAddressId());

        serviceOrderVO.setAddress(serviceAddressVO.getAddress());
        serviceOrderVO.setName(serviceOrderVO.getName());
        serviceOrderVO.setPhone(serviceOrderVO.getPhone());

        return serviceOrderVO;
    }


    public Result<PageInfoVO<ServiceOrderVO>> selectByUserId(PageServiceOrderDTO dto, Long uid) {
        PageHelper.startPage(dto.getPage(), dto.getSize());
        List<ServiceOrder> serviceOrders = serviceOrderBizMapper.selectServiceOrderPage(dto, uid);
        List<ServiceOrderVO> serviceOrderVOList = serviceOrders.parallelStream()
                .map(e -> AttrCopyUtils.copy(e, new ServiceOrderVO())).collect(Collectors.toList());
        return ResultGenerator.success(new PageInfoVO<>(serviceOrderVOList));
    }


    public int save(SaveServiceOrderDTO dto, Long uid) {
        ServiceOrder serviceOrder = AttrCopyUtils.copy(dto, new ServiceOrder());
        // 账户号
        serviceOrder.setAccountNo(uid);
        // 订单号
        serviceOrder.setOrderNo(RandomUtils.orderNum());
        return this.insertSelective(serviceOrder);
    }


    public int valuation(ValuationServiceOrderDTO dto) {
        ServiceOrder serviceOrder = this.selectByPrimaryKey(dto.getId());
        // 找不到订单
        if (Objects.isNull(serviceOrder)) {
            throw new ServiceException(BizCodeEnum.SERVICE_ORDER_NOT_EXIST);
        }

        // 估价价格
        serviceOrder.setPrice(dto.getPrice());
        // TODO 置为以估价状态
        // serviceOrder.setOrderStatus();

        return this.updateByPrimaryKey(serviceOrder);
    }


}
